import qbs
import qbs.FileInfo

Product {
    type: ["application", "hex", "bin", "size", "cp_hex" ]
    Depends { name:"cpp" }

    property string target: "K1921VK01T"
    property string periph_driver: "../../../../../../periph_driver_lib/"

    cpp.debugInformation: true
    cpp.defines: [
        "__STARTUP_CLEAR_BSS",
        "__START=main",
        target,
        "EXT_OSC_VALUE=((uint32_t)16000000)"
    ]
    cpp.optimization: "small"
    cpp.cxxFlags: [
        "-std=c++11"
    ]
    cpp.cFlags: [
        "-std=gnu99"
    ]
    cpp.warningLevel: "all"
    cpp.commonCompilerFlags: [
        "-mthumb",
        "-mcpu=cortex-m4",
        "-mfloat-abi=hard",
        "-mfpu=fpv4-sp-d16",
        "-fdata-sections",
        "-ffunction-sections",
        "-fno-builtin",
        "-flto",
        "-mlong-calls"
    ]

    cpp.positionIndependentCode: false
    cpp.executableSuffix: ".elf"
    cpp.linkerScripts: [
        "ldscripts/" + target + ".ld",
    ]
    cpp.linkerFlags:[
        "-mthumb",
        "-mcpu=cortex-m4",
        "-mfloat-abi=hard",
        "-mfpu=fpv4-sp-d16",
        "--specs=nano.specs",
        "--specs=nosys.specs",
        "-Wl,--gc-sections",
        "-lgcc","-lc",
    ]

    cpp.includePaths: [
        "libs/CMSIS/Include/",
        "libs/Device/",
        periph_driver + "inc/"
    ]

    Group {
        name: "Application"
        prefix: "./"
        files: [
            "*.c",
            "*.cpp",
            "*.h",
            "*.s"
        ]
    }
    Group {
        name: "CMSIS"
        prefix: "libs/CMSIS/**/"
        files: [
            "*.c",
            "*.cpp",
            "*.h",
            "*.s"
        ]
    }
    Group {
        name: "Device"
        prefix: "libs/Device/**/"
        files: [
            "*.c",
            "*.cpp",
            "*.h",
            "*.s"
        ]
    }
    Group {
        name: "Peripheral Driver"
        prefix: periph_driver + "/**/"
        files: [
            "*.c",
            "*.cpp",
            "*.h",
            "*.s"
        ]
    }
    Group {
        name: "Linker Scripts"
        prefix: "ldscripts/**/"
        files: "*.ld"
    }

    Rule {
        id: hex
        inputs: "application"
        Artifact {
            fileTags: ["hex"]
            filePath: FileInfo.baseName(input.filePath) + ".hex"
        }
        prepare: {
            var args = ["-O", "ihex", input.filePath, output.filePath];
            var cmd = new Command("arm-none-eabi-objcopy", args);
            cmd.description = "converting to hex: "+FileInfo.fileName(input.filePath);
            cmd.highlight = "linker";
            return cmd;

        }
    }

    Rule {
        id: bin
        inputs: "application"
        Artifact {
            fileTags: ["bin"]
            filePath: FileInfo.baseName(input.filePath) + ".bin"
        }
        prepare: {
            var args = ["-O", "binary", input.filePath, output.filePath];
            var cmd = new Command("arm-none-eabi-objcopy", args);
            cmd.description = "converting to bin: "+FileInfo.fileName(input.filePath);
            cmd.highlight = "linker";
            return cmd;

        }
    }

    Rule {
        id: size
        inputs: ["application"]
        Artifact {
            fileTags: ["size"]
            filePath: "-"
        }
        prepare: {
            var args = [input.filePath];
            var cmd = new Command("arm-none-eabi-size", args);
            cmd.description = "File size: " + FileInfo.fileName(input.filePath);
            cmd.highlight = "linker";
            return cmd;
        }
    }
}


